import asyncio
import base64
import datetime
import json
import re
import uuid
from decimal import Decimal
from io import BytesIO
from pathlib import Path

try:
    from pydantic import BaseModel
except ImportError:
    BaseModel = None

try:
    from bson import ObjectId
    from bson.decimal128 import Decimal128
except ImportError:
    Decimal128 = None
    ObjectId = None

try:
    import numpy as np
except ImportError:
    np = None

try:
    import pandas as pd
except ImportError:
    pd = None

try:
    from PIL import Image
except ImportError:
    Image = None


class JSONSerializer(json.JSONEncoder):
    def default(self, obj: object) -> str | None:
        serializers = [
            self._serialize_simple,
            self._serialize_bytes,
            self._serialize_bytesio,
            self._serialize_image,
            self._serialize_datetime,
            self._serialize_uuid,
            self._serialize_path,
            self._serialize_to_json,
            self._serialize_pydantic,
            self._serialize_bson,
            self._serialize_exception,
            self._serialize_decimal,
            self._serialize_pandas,
            self._serialize_numpy,
        ]
        for serializer in serializers:
            result = serializer(obj)
            if result is not None:
                return result
        if callable(obj) or asyncio.iscoroutine(obj):
            import warnings

            func_name = getattr(obj, "__name__", str(obj))
            warnings.warn(
                f"Attempting to serialize function/coroutine '{func_name}' "
                "which is not supported",
                stacklevel=2,
            )
            raise ValueError(
                f"Attempting to serialize function/coroutine '{func_name}' "
                "which is not supported"
            )
        return super().default(obj)

    def _serialize_simple(self, obj: object) -> str | None:
        if isinstance(
            obj,
            (
                str,
                int,
                float,
                complex,
                list,
                tuple,
                range,
                dict,
                set,
                frozenset,
                bool,
                type(None),
            ),
        ):
            return str(obj)
        return None

    def _serialize_bytes(
        self, obj: bytes | bytearray | memoryview
    ) -> str | None:
        if isinstance(obj, (bytes, bytearray, memoryview)):
            base64_str = base64.b64encode(bytes(obj)).decode("utf-8")
            return f"data:application/octet-stream;base64,{base64_str}"
        return None

    def _serialize_bytesio(self, obj: BytesIO) -> str | None:
        if isinstance(obj, BytesIO):
            base64_str = base64.b64encode(obj.getvalue()).decode("utf-8")
            return f"data:application/octet-stream;base64,{base64_str}"
        return None

    def _serialize_image(self, obj: object) -> str | None:
        if Image and isinstance(obj, Image.Image):
            buffered = BytesIO()
            template = getattr(obj, "format", "PNG") or "PNG"
            obj.save(buffered, format=template)
            base64_str = base64.b64encode(buffered.getvalue()).decode("utf-8")
            return f"data:image/{template.lower()};base64,{base64_str}"
        return None

    def _serialize_datetime(
        self, obj: datetime.datetime | datetime.date | datetime.time
    ) -> str | None:
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        if isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        if isinstance(obj, datetime.time):
            return obj.strftime("%H:%M:%S.%f")
        return None

    def _serialize_uuid(self, obj: uuid.UUID) -> str | None:
        if isinstance(obj, uuid.UUID):
            return str(obj)
        return None

    def _serialize_path(self, obj: Path) -> str | None:
        if isinstance(obj, Path):
            return str(obj)
        return None

    def _serialize_to_json(self, obj: object) -> str | None:
        if hasattr(obj, "to_json"):
            return obj.to_json()
        return None

    def _serialize_pydantic(self, obj: object) -> str | None:
        if BaseModel and isinstance(obj, BaseModel):
            return obj.model_dump()
        return None

    def _serialize_bson(self, obj: object) -> str | None:
        if ObjectId and isinstance(obj, ObjectId):
            return str(obj)
        if Decimal128 and isinstance(obj, Decimal128):
            return str(obj)
        return None

    def _serialize_exception(self, obj: Exception) -> str | None:
        if isinstance(obj, Exception):
            return repr(obj)
        return None

    def _serialize_decimal(self, obj: Decimal) -> str | None:
        if isinstance(obj, Decimal):
            return str(obj)
        return None

    def _serialize_pandas(self, obj: object) -> str | None:
        if pd and isinstance(obj, pd.Series):
            return obj.to_dict()
        return None

    def _serialize_numpy(self, obj: object) -> str | None:
        if np and isinstance(obj, np.ndarray):
            return obj.tolist()
        return None


def _try_parse_json_string(value: str | object) -> object:
    if not isinstance(value, str):
        return value
    if value.startswith("{") and value.endswith("}"):
        try:
            return loads(value)
        except json.JSONDecodeError:
            pass
    return value


def _try_parse_data_url(value: str | object) -> object:
    if not isinstance(value, str):
        return value
    if value.startswith("data:"):
        try:
            mime_type, b64data = value[5:].split(";base64,", 1)
            binary_data = base64.b64decode(b64data)
            if mime_type.startswith("image/") and Image:
                return Image.open(BytesIO(binary_data))
            elif mime_type == "application/octet-stream":
                return binary_data
        except (ValueError, TypeError):
            pass
    return value


def _try_parse_uuid(value: str | object) -> object:
    if not isinstance(value, str):
        return value
    if re.match(
        r"^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-"
        r"[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$",
        value,
    ):
        try:
            return uuid.UUID(value)
        except ValueError:
            pass
    return value


def _try_parse_datetime(value: str | object) -> object:
    if not isinstance(value, str):
        return value
    datetime_patterns = [
        # ISO format with timezone
        (
            r"^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d+)?(?:Z|[+-]\d{2}:?\d{2})?$",
            lambda v: datetime.datetime.fromisoformat(
                v.replace("Z", "+00:00")
            ),
        ),
        # Date and time
        (
            r"^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}(?:\.\d+)?$",
            lambda v: datetime.datetime.strptime(
                v,
                "%Y-%m-%d %H:%M:%S.%f" if "." in v else "%Y-%m-%d %H:%M:%S",
            ),
        ),
        # Date only
        (
            r"^\d{4}-\d{2}-\d{2}$",
            lambda v: datetime.datetime.strptime(v, "%Y-%m-%d").date(),
        ),
        # Time with microseconds
        (
            r"^\d{2}:\d{2}:\d{2}\.\d+$",
            lambda v: datetime.datetime.strptime(v, "%H:%M:%S.%f").time(),
        ),
        # Time without microseconds
        (
            r"^\d{2}:\d{2}:\d{2}$",
            lambda v: datetime.datetime.strptime(v, "%H:%M:%S").time(),
        ),
    ]
    for pattern, parser in datetime_patterns:
        if re.match(pattern, value):
            try:
                return parser(value)
            except ValueError:
                continue
    return value


def json_deserializer(dct: dict) -> dict:
    for key, value in dct.items():
        if isinstance(value, str):
            # Try to parse JSON strings that might be from pandas Series
            value = _try_parse_json_string(value)
            # Data URL decoding
            # (handles bytes, bytearray, memoryview, BytesIO, and Images)
            value = _try_parse_data_url(value)
            # Non-serializable function references (skip conversion)
            if isinstance(value, str) and value.startswith(
                "<non-serializable-function:"
            ):
                dct[key] = value
                continue
            # UUID conversion
            value = _try_parse_uuid(value)
            # Datetime patterns
            value = _try_parse_datetime(value)
            dct[key] = value
    return dct


def dumps(*args: list, **kwargs: dict) -> str:
    kwargs.setdefault("cls", JSONSerializer)
    return json.dumps(*args, **kwargs)


def dump(*args: list, **kwargs: dict) -> None:
    kwargs.setdefault("cls", JSONSerializer)
    return json.dump(*args, **kwargs)


def loads(*args: list, **kwargs: dict) -> dict:
    kwargs.setdefault("object_hook", json_deserializer)
    return json.loads(*args, **kwargs)


def load(*args: list, **kwargs: dict) -> dict:
    kwargs.setdefault("object_hook", json_deserializer)
    return json.load(*args, **kwargs)
